{                                                                          }
{    General imports from the GPC Dos unit for the Sysutils unit           }
{                                                                          }
{    Copyright (C) 1997-2001 Free Software Foundation, Inc.                }
{                                                                          }
{    Author:                                                               }
{       Frank Heckenbach <frank@g-n-u.de>                                  }
{                                                                          }
{    This file is part of GNU Pascal.                                      }
{
     Routines imported from the Dos unit
         UnpackTime
         PackTime
         GetDate
         GetTime
         DiskFree
         DiskSize
         GetFAttr
         SetFAttr
         GetFTime
         SetFtime
}
{                                                                          }
{                                                                          }
{    This library is a free library; you can redistribute and/or modify    }
{    it under the terms of the GNU Library General Public License,         }
{    version 2, as published by the Free Software Foundation.              }
{                                                                          }
{    This library is distributed in the hope that it will be useful,       }
{    but WITHOUT ANY WARRANTY; without even the implied warranty of        }
{    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         }
{    GNU Library General Public License for more details.                  }
{                                                                          }
{    You should have received a copy of the GNU Library General Public     }
{    License along with this library (see the file COPYING.LIB);           }
{    if not, write to the Free Software Foundation, Inc., 675 Mass Ave,    }
{    Cambridge, MA 02139, USA.                                             }
{                                                                          }
{    As a special exception, if you link this file with files compiled     }
{    with a GNU compiler to produce an executable, this does not cause     }
{    the resulting executable to be covered by the GNU Library General     }
{    Public License. This exception does not however invalidate any other  }
{    reasons why the executable file might be covered by the GNU Library   }
{    General Public License.                                               }
{                                                                          }

{$B-,I-}

VAR
  { Error status variable }
  DosError : Integer = 0;

CONST
  { File attribute constants }
  ReadOnly      = $01;
  Hidden        = $02; { set for dot files except '.' and '..' }
  SysFile       = $04; { not supported }
  VolumeID      = $08; { not supported }
  Directory     = $10;
  Archive       = $20; { means: not executable }
  {$local W-} AnyFile = $3F; {$endlocal}


  { DosError codes }
  DosError_FileNotFound = 2;
  DosError_PathNotFound = 3;
  DosError_AccessDenied = 5;
  DosError_InvalidMem   = 9;
  DosErorr_InvalidEnv   = 10;
  DosError_NoMoreFiles  = 18;
  DosError_IOError      = 29;
  DosError_ReadFault    = 30;

TYPE
  PLongInt = ^LongInt;

VAR
  DosExitCodeVar : Word = 0;
  TimeDelta : MicroSecondTimeType = 0;

PROCEDURE UnpackTime ( P : LongInt; VAR T : DateTime ); FORWARD;
PROCEDURE PackTime ( CONST T : DateTime; VAR P : LongInt ); FORWARD;

PROCEDURE GetDate ( VAR Year, Month, Day, DayOfWeek : Word );
VAR
  t : MicroSecondTimeType;
  ts : TimeStamp;
BEGIN
  t := GetMicroSecondTime + TimeDelta;
  UnixTimeToTimeStamp ( t div 1000000, ts );
  Year      := ts.Year;
  Month     := ts.Month;
  Day       := ts.Day;
  DayOfWeek := ts.DayOfWeek
END;

PROCEDURE GetTime ( VAR Hour, Minute, Second, Sec100 : Word );
VAR
  t : MicroSecondTimeType;
  ts : TimeStamp;
BEGIN
  t := GetMicroSecondTime + TimeDelta;
  UnixTimeToTimeStamp ( t div 1000000, ts );
  Hour   := ts.Hour;
  Minute := ts.Minute;
  Second := ts.Second;
  Sec100 := ( t mod 1000000 ) div 10000
END;

FUNCTION DiskFree ( Drive : Byte ) : LongInt;
VAR
  Path : String ( 2 );
  Buf : StatFSBuffer;
BEGIN
  IF Drive = 0 THEN
    Path := DirSelf
  ELSE
    Path := Succ ( 'a', Drive - 1 ) + ':';
  StatFS ( Path, Buf );
  IF IOResult = 0 THEN
    DiskFree := Buf.BlockSize * Buf.BlocksFree
  ELSE
    BEGIN
      DosError := DosError_AccessDenied;
      DiskFree := - 1
    END
END;

FUNCTION DiskSize ( Drive : Byte ) : LongInt;
VAR
  Path : String ( 2 );
  Buf : StatFSBuffer;
BEGIN
  IF Drive = 0 THEN
    Path := DirSelf
  ELSE
    Path := Succ ( 'a', Drive - 1 ) + ':';
  StatFS ( Path, Buf );
  IF IOResult = 0 THEN
    DiskSize := Buf.BlockSize * Buf.BlocksTotal
  ELSE
    BEGIN
      DosError := DosError_AccessDenied;
      DiskSize := - 1
    END
END;

PROCEDURE GetFAttr ( VAR F : GPC_AnyFile; VAR Attr : TDosAttr );
VAR
  b : BindingType;
  s : TString;
  d : OrigInt;
BEGIN
  b := Binding ( F );
  Attr := 0;
  IF not ( b.Bound AND ( b.Existing OR b.Directory OR b.Special ) ) THEN
    DosError := DosError_FileNotFound
  ELSE
    BEGIN
      DosError := 0;
      IF b.Directory      THEN Attr := Attr OR Directory;
      IF not b.Writable   THEN Attr := Attr OR ReadOnly;
      IF not b.Executable THEN Attr := Attr OR Archive;
      d := Length ( b.Name );
      WHILE ( d > 0 ) AND not ( b.Name [d] IN DirSeparators ) DO Dec ( d );
      IF ( Length ( b.Name ) > d + 1 ) AND ( b.Name [d + 1] =  '.' ) AND
        ( ( Length ( b.Name ) > d + 2 ) OR  ( b.Name [d + 2] <> '.' ) ) THEN
        Attr := Attr OR Hidden
    END
END;

PROCEDURE SetFAttr ( VAR F : GPC_AnyFile; Attr : TDosAttr );
VAR b : BindingType;
BEGIN
  b := Binding ( F );
  IF not b.Bound THEN
    BEGIN
      DosError := DosError_FileNotFound;
      Exit
    END;
  IF Attr AND ReadOnly = 0
    THEN OR  ( b.Mode, fm_UserWritable ) { Set only user write permissions, for reasons of safety! }
    ELSE AND ( b.Mode, not ( fm_UserWritable OR fm_GroupWritable OR fm_OthersWritable ) );
  IF Attr AND Archive = 0
    THEN OR  ( b.Mode, fm_UserExecutable OR fm_GroupExecutable OR fm_OthersExecutable )
    ELSE AND ( b.Mode, not ( fm_UserExecutable OR fm_GroupExecutable OR fm_OthersExecutable ) );
  ChMod ( F, b.Mode );
  IF IOResult <> 0 THEN DosError := DosError_AccessDenied
END;

PROCEDURE GetFTime ( VAR F : GPC_AnyFile; VAR aTime : LongInt );
VAR
  b : BindingType;
  Year, Month, Day, Hour, Minute, Second : OrigInt;
  dt : DateTime;
BEGIN
  b := Binding ( F );
  IF not ( b.Bound AND ( b.Existing OR b.Directory OR b.Special ) ) THEN
    DosError := DosError_FileNotFound
  ELSE
    BEGIN
      IF b.ModificationTime >= 0 THEN
        BEGIN
          UnixTimeToTime ( b.ModificationTime, Year, Month, Day, Hour, Minute, Second );
          dt.Year  := Year;
          dt.Month := Month;
          dt.Day   := Day;
          dt.Hour  := Hour;
          dt.Min   := Minute;
          dt.Sec   := Second;
          PackTime ( dt, aTime )
        END
      ELSE
        aTime := 0;
      DosError := 0
    END
END;

PROCEDURE SetFTime ( VAR F : GPC_AnyFile; aTime : LongInt );
VAR
  dt : DateTime;
  ut : UnixTimeType;
BEGIN
  UnpackTime ( aTime, dt );
  WITH dt DO ut := TimeToUnixTime ( Year, Month, Day, Hour, Min, Sec );
  DosError := DosError_AccessDenied;
  IF ut >= 0 THEN
    BEGIN
      SetFileTime ( F, ut, ut );
      IF IOResult = 0 THEN DosError := 0
    END
END;

PROCEDURE UnpackTime ( P : LongInt; VAR T : DateTime );
BEGIN
  T.Year  := ( P SHR 25 ) AND $7f + 1980;
  T.Month := ( P SHR 21 ) AND $f;
  T.Day   := ( P SHR 16 ) AND $1f;
  T.Hour  := ( P SHR 11 ) AND $1f;
  T.Min   := ( P SHR 5 ) AND $3f;
  T.Sec   := 2 * ( P AND $1f )
END;

PROCEDURE PackTime ( CONST T : DateTime; VAR P : LongInt );
BEGIN
  P := ( LongInt ( T.Year ) - 1980 ) SHL 25 + LongInt ( T.Month ) SHL 21 + LongInt ( T.Day ) SHL 16
       + T.Hour SHL 11 + T.Min SHL 5 + T.Sec div 2
END;


